define("ace/mode/plain_text", ["require", "exports", "module", "ace/lib/oop", "ace/mode/text", "ace/mode/text_highlight_rules", "ace/mode/behaviour"], function (require, exports, module) {
    "use strict";

    var oop = require("../lib/oop");
    var TextMode = require("./text").Mode;
    var TextHighlightRules = require("./text_highlight_rules").TextHighlightRules;
    var Behaviour = require("./behaviour").Behaviour;

    var Mode = function () {
        this.HighlightRules = TextHighlightRules;
        this.$behaviour = new Behaviour();
    };

    oop.inherits(Mode, TextMode);

    (function () {
        this.type = "text";
        this.getNextLineIndent = function (state, line, tab) {
            return '';
        };
        this.$id = "ace/mode/plain_text";
    }).call(Mode.prototype);

    exports.Mode = Mode;
});
