define("ace/mode/folding/cstyle", ["require", "exports", "module", "ace/lib/oop", "ace/range", "ace/mode/folding/fold_mode"], function (require, exports, module) {
    "use strict";

    var oop = require("../../lib/oop");
    var Range = require("../../range").Range;
    var BaseFoldMode = require("./fold_mode").FoldMode;

    var FoldMode = exports.FoldMode = function (commentRegex) {
        if (commentRegex) {
            this.foldingStartMarker = new RegExp(
                this.foldingStartMarker.source.replace(/\|[^|]*?$/, "|" + commentRegex.start)
            );
            this.foldingStopMarker = new RegExp(
                this.foldingStopMarker.source.replace(/\|[^|]*?$/, "|" + commentRegex.end)
            );
        }
    };
    oop.inherits(FoldMode, BaseFoldMode);

    (function () {

        this.foldingStartMarker = /(\{|\[)[^\}\]]*$|^\s*(\/\*)/;
        this.foldingStopMarker = /^[^\[\{]*(\}|\])|^[\s\*]*(\*\/)/;

        this.getFoldWidgetRange = function (session, foldStyle, row, forceMultiline) {
            var line = session.getLine(row);
            var match = line.match(this.foldingStartMarker);
            if (match) {
                var i = match.index;

                if (match[1])
                    return this.openingBracketBlock(session, match[1], row, i);

                var range = session.getCommentFoldRange(row, i + match[0].length, 1);

                if (range && !range.isMultiLine()) {
                    if (forceMultiline) {
                        range = this.getSectionRange(session, row);
                    } else if (foldStyle != "all")
                        range = null;
                }

                return range;
            }

            if (foldStyle === "markbegin")
                return;

            var match = line.match(this.foldingStopMarker);
            if (match) {
                var i = match.index + match[0].length;

                if (match[1])
                    return this.closingBracketBlock(session, match[1], row, i);

                return session.getCommentFoldRange(row, i, -1);
            }
        };

        this.getSectionRange = function (session, row) {
            var line = session.getLine(row);
            var startIndent = line.search(/\S/);
            var startRow = row;
            var startColumn = line.length;
            row = row + 1;
            var endRow = row;
            var maxRow = session.getLength();
            while (++row < maxRow) {
                line = session.getLine(row);
                var indent = line.search(/\S/);
                if (indent === -1)
                    continue;
                if (startIndent > indent)
                    break;
                var subRange = this.getFoldWidgetRange(session, "all", row);

                if (subRange) {
                    if (subRange.start.row <= startRow) {
                        break;
                    } else if (subRange.isMultiLine()) {
                        row = subRange.end.row;
                    } else if (startIndent == indent) {
                        break;
                    }
                }
                endRow = row;
            }

            return new Range(startRow, startColumn, endRow, session.getLine(endRow).length);
        };

    }).call(FoldMode.prototype);

});

define("ace/mode/tcl_highlight_rules", ["require", "exports", "module", "ace/lib/oop", "ace/mode/text_highlight_rules"], function (require, exports, module) {
    "use strict";

    var oop = require("../lib/oop");
    var TextHighlightRules = require("./text_highlight_rules").TextHighlightRules;

    var TclHighlightRules = function () {

        this.$rules = {
            "start": [
                {
                    token: "comment",
                    regex: "#.*\\\\$",
                    next: "commentfollow"
                }, {
                    token: "comment",
                    regex: "#.*$"
                }, {
                    token: "support.function",
                    regex: '[\\\\]$',
                    next: "splitlineStart"
                }, {
                    token: "text",
                    regex: '[\\\\](?:["]|[{]|[}]|[[]|[]]|[$]|[\])'
                }, {
                    token: "text", // last value before command
                    regex: '^|[^{][;][^}]|[/\r/]',
                    next: "commandItem"
                }, {
                    token: "string", // single line
                    regex: '[ ]*["](?:(?:\\\\.)|(?:[^"\\\\]))*?["]'
                }, {
                    token: "string", // multi line """ string start
                    regex: '[ ]*["]',
                    next: "qqstring"
                }, {
                    token: "variable.instance",
                    regex: "[$]",
                    next: "variable"
                }, {
                    token: "support.function",
                    regex: "!|\\$|%|&|\\*|\\-\\-|\\-|\\+\\+|\\+|~|===|==|=|!=|!==|<=|>=|<<=|>>=|>>>=|<>|<|>|!|&&|\\|\\||\\?\\:|\\*=|%=|\\+=|\\-=|&=|\\^=|{\\*}|;|::"
                }, {
                    token: "identifier",
                    regex: "[a-zA-Z_$][a-zA-Z0-9_$]*\\b"
                }, {
                    token: "paren.lparen",
                    regex: "[[{]",
                    next: "commandItem"
                }, {
                    token: "paren.lparen",
                    regex: "[(]"
                }, {
                    token: "paren.rparen",
                    regex: "[\\])}]"
                }, {
                    token: "text",
                    regex: "\\s+"
                }
            ],
            "commandItem": [
                {
                    token: "comment",
                    regex: "#.*\\\\$",
                    next: "commentfollow"
                }, {
                    token: "comment",
                    regex: "#.*$",
                    next: "start"
                }, {
                    token: "string", // single line
                    regex: '[ ]*["](?:(?:\\\\.)|(?:[^"\\\\]))*?["]'
                }, {
                    token: "variable.instance",
                    regex: "[$]",
                    next: "variable"
                }, {
                    token: "support.function",
                    regex: "(?:[:][:])[a-zA-Z0-9_/]+(?:[:][:])",
                    next: "commandItem"
                }, {
                    token: "support.function",
                    regex: "[a-zA-Z0-9_/]+(?:[:][:])",
                    next: "commandItem"
                }, {
                    token: "support.function",
                    regex: "(?:[:][:])",
                    next: "commandItem"
                }, {
                    token: "paren.rparen",
                    regex: "[\\])}]"
                }, {
                    token: "support.function",
                    regex: "!|\\$|%|&|\\*|\\-\\-|\\-|\\+\\+|\\+|~|===|==|=|!=|!==|<=|>=|<<=|>>=|>>>=|<>|<|>|!|&&|\\|\\||\\?\\:|\\*=|%=|\\+=|\\-=|&=|\\^=|{\\*}|;|::"
                }, {
                    token: "keyword",
                    regex: "[a-zA-Z0-9_/]+",
                    next: "start"
                }],
            "commentfollow": [
                {
                    token: "comment",
                    regex: ".*\\\\$",
                    next: "commentfollow"
                }, {
                    token: "comment",
                    regex: '.+',
                    next: "start"
                }],
            "splitlineStart": [
                {
                    token: "text",
                    regex: "^.",
                    next: "start"
                }],
            "variable": [
                {
                    token: "variable.instance", // variable tcl
                    regex: "[a-zA-Z_\\d]+(?:[(][a-zA-Z_\\d]+[)])?",
                    next: "start"
                }, {
                    token: "variable.instance", // variable tcl with braces
                    regex: "{?[a-zA-Z_\\d]+}?",
                    next: "start"
                }],
            "qqstring": [{
                token: "string", // multi line """ string end
                regex: '(?:[^\\\\]|\\\\.)*?["]',
                next: "start"
            }, {
                token: "string",
                regex: '.+'
            }]
        };
    };

    oop.inherits(TclHighlightRules, TextHighlightRules);

    exports.TclHighlightRules = TclHighlightRules;
});

define("ace/mode/matching_brace_outdent", ["require", "exports", "module", "ace/range"], function (require, exports, module) {
    "use strict";

    var Range = require("../range").Range;

    var MatchingBraceOutdent = function () {
    };

    (function () {

        this.checkOutdent = function (line, input) {
            if (!/^\s+$/.test(line))
                return false;

            return /^\s*\}/.test(input);
        };

        this.autoOutdent = function (doc, row) {
            var line = doc.getLine(row);
            var match = line.match(/^(\s*\})/);

            if (!match) return 0;

            var column = match[1].length;
            var openBracePos = doc.findMatchingBracket({row: row, column: column});

            if (!openBracePos || openBracePos.row == row) return 0;

            var indent = this.$getIndent(doc.getLine(openBracePos.row));
            doc.replace(new Range(row, 0, row, column - 1), indent);
        };

        this.$getIndent = function (line) {
            return line.match(/^\s*/)[0];
        };

    }).call(MatchingBraceOutdent.prototype);

    exports.MatchingBraceOutdent = MatchingBraceOutdent;
});

define("ace/mode/tcl", ["require", "exports", "module", "ace/lib/oop", "ace/mode/text", "ace/mode/folding/cstyle", "ace/mode/tcl_highlight_rules", "ace/mode/matching_brace_outdent", "ace/range"], function (require, exports, module) {
    "use strict";

    var oop = require("../lib/oop");
    var TextMode = require("./text").Mode;
    var CStyleFoldMode = require("./folding/cstyle").FoldMode;
    var TclHighlightRules = require("./tcl_highlight_rules").TclHighlightRules;
    var MatchingBraceOutdent = require("./matching_brace_outdent").MatchingBraceOutdent;
    var Range = require("../range").Range;

    var Mode = function () {
        this.HighlightRules = TclHighlightRules;
        this.$outdent = new MatchingBraceOutdent();
        this.foldingRules = new CStyleFoldMode();
    };
    oop.inherits(Mode, TextMode);

    (function () {

        this.lineCommentStart = "#";

        this.getNextLineIndent = function (state, line, tab) {
            var indent = this.$getIndent(line);

            var tokenizedLine = this.getTokenizer().getLineTokens(line, state);
            var tokens = tokenizedLine.tokens;

            if (tokens.length && tokens[tokens.length - 1].type == "comment") {
                return indent;
            }

            if (state == "start") {
                var match = line.match(/^.*[\{\(\[]\s*$/);
                if (match) {
                    indent += tab;
                }
            }

            return indent;
        };

        this.checkOutdent = function (state, line, input) {
            return this.$outdent.checkOutdent(line, input);
        };

        this.autoOutdent = function (state, doc, row) {
            this.$outdent.autoOutdent(doc, row);
        };

        this.$id = "ace/mode/tcl";
    }).call(Mode.prototype);

    exports.Mode = Mode;
});
